@@ -46,7 +46,7 @@ from contextlib import contextmanager
 import msgpack
 import pylibmc
 
-VERSION = "0.7.0"
+VERSION = "0.7.1"
 LOCK_SH = 0
 LOCK_EX = 1
 RUN_PATH = "/run/starrydb/"
@@ -300,8 +300,9 @@ class StarryDepot(object):
         self.cache = None
         self.fsync = False
         self.bak = False
-        self.thread_cond = threading.Condition()
-        self.thread_lock = threading.RLock()
+        self.depot_lock = threading.Condition()
+        self.thread_lock = threading.Condition()
+        self.thread_blocking = False
         self.write_wait = 0
         self.read_wait = 0
         self.write_queue = []
@@ -311,43 +312,49 @@ class StarryDepot(object):
             raise StarryError(f"path {path} doesn't exist.")
 
     def executeCommand(self, conn, mode=LOCK_SH):
-        if mode == LOCK_SH:
-            while True:
-                try:
-                    (func, *param) = conn.recv()
-                except:
-                    break
+        def _threadPredicate():
+            return not self.thread_blocking
 
-                try:
-                    conn.send(getattr(self, func)(*param))
-                except Exception as e:
-                    self.logger.debug(f"Caught exception: {e.__doc__}({e})")
+        def _depotPredicate():
+            if self.read_wait > 0 or self.write_wait > 0 or len(self.read_queue) > 0 or len(self.write_queue) > 0:
+                return False
 
-                    try:
-                        conn.send((False, f"Caught exception: {e.__doc__}({e})"))
-                    except:
-                        pass
+            return True
 
-                    break
-        else:
+        if mode == LOCK_EX:
             with self.thread_lock:
-                while True:
-                    try:
-                        (func, *param) = conn.recv()
-                    except:
-                        break
+                self.thread_lock.wait_for(_threadPredicate)
+                self.thread_blocking = True
 
-                    try:
-                        conn.send(getattr(self, func)(*param))
-                    except Exception as e:
-                        self.logger.debug(f"Caught exception: {e.__doc__}({e})")
+        while True:
+            try:
+                (func, *param) = conn.recv()
+            except:
+                break
 
-                        try:
-                            conn.send((False, f"Caught exception: {e.__doc__}({e})"))
-                        except:
-                            pass
+            if mode == LOCK_SH:
+                with self.thread_lock:
+                    self.thread_lock.wait_for(_threadPredicate)
+            else:
+                with self.depot_lock:
+                    self.depot_lock.wait_for(_depotPredicate)
 
-                        break
+            try:
+                conn.send(getattr(self, func)(*param))
+            except Exception as e:
+                self.logger.debug(f"Caught exception: {e.__doc__}({e})")
+
+                try:
+                    conn.send((False, f"Caught exception: {e.__doc__}({e})"))
+                except:
+                    pass
+
+                break
+
+        if mode == LOCK_EX:
+            with self.thread_lock:
+                self.thread_blocking = False
+                self.thread_lock.notify_all()
 
         if self.cache is not None:
             try:
@@ -1096,13 +1103,13 @@ class StarryDepot(object):
 
             return True
 
-        with self.thread_cond:
+        with self.depot_lock:
             if mark["mode"] == "r":
                 self.read_wait += 1
             else:
                 self.write_wait += 1
 
-            self.thread_cond.wait_for(_predicate)
+            self.depot_lock.wait_for(_predicate)
 
             if mark["mode"] == "r":
                 self.read_queue.append(mark["members"])
@@ -1113,7 +1120,7 @@ class StarryDepot(object):
 
         yield
 
-        with self.thread_cond:
+        with self.depot_lock:
             try:
                 if mark["mode"] == "r":
                     self.read_queue.remove(mark["members"])
@@ -1122,7 +1129,7 @@ class StarryDepot(object):
             except:
                 pass
 
-            self.thread_cond.notify_all()
+            self.depot_lock.notify_all()
 
 class StarryClient(object):
     def __init__(self, path, mode=LOCK_SH, address=SERVER_ADDRESS):
